/**@file cs1dCGbasic_main.cpp
 * @brief main file for the cs1d example (basic version without plugins)
 * @author Luigi De Giovanni
 */

#include "master.hpp"
#include "slave.hpp"
#include <cstdlib>
#include <iostream>
#include "scip_exception.hpp"

using namespace std;
using namespace scipMeMOC;


/** main function for cs1d tutorial */
int
main(
     int args,
     char ** argv
     )
{
   cout << "******************************************" << endl;
   cout << "* cs1d solver based on SCIP (no plugins) *" << endl;
   cout << "*                                        *" << endl;
   cout << "* (c) Luigi De Giovanni (2009)           *" << endl;
   cout << "******************************************" << endl << endl;

   if (args < 2)
   {
      cerr << "call " << argv[0] << " <data file>" << endl;
      exit(1);
   }

   try
   {
      cout << "*** SOLVING CONTINUOUS RELAXATION ***" << endl;
      
      // initialize the master
      CSMaster master(argv[1]);
      
      // generate columns (cutting patterns)
      SCIP_Real slaveObj = 0.0;
      int maxiter = 100;
      do {
        cout << "-------------- iter" << -maxiter << " ----------------------" << endl;
	// solve the master problem
	master.solveLP();
	// display the solution on stdout
	//master.display();
	
	//solve the slave
	CSSlave slave(master.getDualValues(),master.getWidths(),master.getRollWidth());
	slave.solve();
	//slave.display();
	
        std::vector<int> newPattern(master.getWidths().size());
	slaveObj = slave.getSolution(newPattern);
	if (slaveObj > 1) {
          master.addPattern(newPattern);
        }
	maxiter--;
      } while (slaveObj > 1 && maxiter > 0);
      
      cout << "\n\n*** SOLVING TO INTEGRALITY ***" << endl;
      master.solveInteger();
      master.display();
      
   } catch(SCIPException& exc)
   {
      cerr << exc.what() << endl;
      exit(exc.getRetcode());
   }
   return EXIT_SUCCESS;
}

